//###########################################################################
// FILE:   blinky_c28.c
// TITLE:  Blinky Example for F28M35x.
//
// Dual Core Blinky Example.  This example demonstrates how to run a 
// implement a standalone application on both cores.  Due to an errate in 
// the bootROM this example may not run correctly with the debugger connected.
// To run the example, program both cores with their respective project and 
// then disconnect the debugger.  Set SW3 switch 1 to the down position
// (disconnect TRSTn) as well as setting all 4 switches on SW1 to the down 
// position.  Cycle power and both LEDs should begin to blink.
//
//###########################################################################
// $TI Release: F28M35x Support Library v150 $
// $Release Date: Mon Sep 17 09:14:32 CDT 2012 $
//###########################################################################

#include "DSP28x_Project.h"     // Device Headerfile and Examples Include File
#include <string.h>
#include "F28M35x_Ipc_drivers.h"
// add by me
#define C28Update (void   (*)(void))0x0013e000  //0013a000
#define IPC_Update_COM   0x00003238

#define C28_FREQ    150         //CPU frequency in MHz

#ifdef _FLASH
// These are defined by the linker (see device linker command file)
extern Uint16 RamfuncsLoadStart;
extern Uint16 RamfuncsLoadSize;
extern Uint16 RamfuncsRunStart;
#endif

void main(void)
{
   unsigned long delay;
// Step 1. Initialize System Control:
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the F28M35x_SysCtrl.c file.
    InitSysCtrl();

// Step 2. Initialize GPIO:
// This example function is found in the F28M35x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
    InitGpio(); // Skipped for this example
    EALLOW;
    GpioG1CtrlRegs.GPCDIR.bit.GPIO70 = 1; 
    EDIS;
    GpioG1DataRegs.GPCDAT.bit.GPIO70 = 1;// turn off LED
// Step 3. Clear all interrupts and initialize PIE vector table:
// Disable CPU interrupts
    DINT;

#ifdef _FLASH    
// Copy time critical code and Flash setup code to RAM
// This includes the following functions: InitFlash();
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the device .cmd file.
   memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize); 
   
// Call Flash Initialization to setup flash wait states
// This function must reside in RAM
    InitFlash();       
#endif

// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
// This function is found in the F28M35x_PieCtrl.c file.
    InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags:
    IER = 0x0000;
    IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in F28M35x_DefaultIsr.c.
// This function is found in F28M35x_PieVect.c.
    InitPieVectTable();
   // for(delay = 0; delay < 20; delay++){}

// Enable global Interrupts and higher priority real-time debug events:
    EINT;  // Enable Global interrupt INTM
    ERTM;  // Enable Global real time interrupt DBGM

// Step 6. IDLE loop. Just sit and loop forever (optional):
    for(;;)
    {
        // C28 poll for IPC_FLAG1 that M3 send command
    	if((CtoMIpcRegs.MTOCIPCSTS.all & IPC_FLAG1))
    	{
    	  if (CtoMIpcRegs.MTOCIPCCOM == IPC_Update_COM)
    	  {
    		  // acknowledge flag1, and flag32 acknowledge in C28 CKFA
    		  CtoMIpcRegs.MTOCIPCACK.all |= (IPC_FLAG1);
    		  // jump to C28 CKFA entry point
    		  (*C28Update)();
    	  }
    	}

    	// Turn on LED
    	//
        GpioG1DataRegs.GPCDAT.bit.GPIO70 = 0;
        // Delay for a bit.
        for(delay = 0; delay < 2000000; delay++)
        {
        }
        // Turn off LED
        GpioG1DataRegs.GPCDAT.bit.GPIO70 = 1;
        // Delay for a bit.
        for(delay = 0; delay < 2000000; delay++)
        {
        }
    
    }

}




