//###########################################################################
// FILE:   uart_echo.c
// TITLE:  Example for reading data from and writing data to the UART in
//         an interrupt driven fashion.
//###########################################################################
// $TI Release: F28M35x Support Library v140 $
// $Release Date: Tue Jul  3 09:38:11 CDT 2012 $
//###########################################################################

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_flash.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "driverlib/device.h"
#include "driverlib/dcsm.h"
#include "driverlib/ipc.h"
#include "Types.h"

#include "F021_CONCERTO_Cortex.h"
#include "driverlib/flash.h"
#include <string.h>
extern unsigned long RamfuncsLoadStart;
extern unsigned long RamfuncsRunStart;
extern unsigned long RamfuncsLoadSize;

#define Update (void   (*)(void))0x00200071  //  0x00204001

//*****************************************************************************
//! \addtogroup master_example_list
//! <h1>UART Echo (uart_echo)</h1>
//!
//! This example application utilizes the UART to echo text.  The first UART
//! (connected to the FTDI virtual serial port on the evaluation board) will be
//! configured in 115,200 baud, 8-n-1 mode.  All characters received on the
//! UART are transmitted back to the UART.
//*****************************************************************************

//*****************************************************************************
// The error routine that is called if the driver library encounters an error.
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
}

#endif

//*****************************************************************************
// The UART interrupt handler.
//*****************************************************************************
void
UARTIntHandler(void)
{
    unsigned long ulStatus;

    // Get the interrrupt status.

    ulStatus = UARTIntStatus(UART0_BASE, true);

    // Clear the asserted interrupts.
    UARTIntClear(UART0_BASE, ulStatus);

    //FIXME Loop while there are characters in the receive FIFO.
 /*   while(UARTCharsAvail(UART0_BASE))
    {
        // Read the next character from the UART and write it back to the UART.
        UARTCharPutNonBlocking(UART0_BASE,
                               UARTCharGetNonBlocking(UART0_BASE));
    }*/
}

//*****************************************************************************
// Send a string to the UART.
//*****************************************************************************
void
UARTSend(const unsigned char *pucBuffer, unsigned long ulCount)
{
    // Loop while there are more characters to send.
    while(ulCount--)
    {
        // Write the next character to the UART.
        UARTCharPutNonBlocking(UART0_BASE, *pucBuffer++);
    }
}

uint16 UART_GetWordData()
{
   uint16 wordData;
   uint16 byteData;

   wordData = 0x0000;
   byteData = 0x0000;

   // Fetch the MSB from the host
   while(UARTCharsAvail(UART0_BASE)!= 1) { }
   byteData =  UARTCharGetNonBlocking(UART0_BASE);

   // Fetch the LSB from the host
   while(UARTCharsAvail(UART0_BASE)!= 1) { }
   wordData =  UARTCharGetNonBlocking(UART0_BASE);


   // form the wordData from the MSB:LSB
   wordData |= (byteData << 8);


   return wordData;
}

unsigned char receiveMSG[200];

//*****************************************************************************
// This example demonstrates how to send a string of data to the UART.
//*****************************************************************************
int
main(void)
{
  //  unsigned int ii=0;
	uint16 wordData = 0x0000;
	uint16 byteData = 0x0000;

    volatile unsigned long ulLoop;
    // Disable Protection
    HWREG(SYSCTL_MWRALLOW) =  0xA5A5A5A5;

    // Setup main clock tree for 75MHz - M3 and 150MHz - C28x
    SysCtlClockConfigSet(SYSCTL_SYSDIV_1 | SYSCTL_M3SSDIV_2 | SYSCTL_USE_PLL |
                         (SYSCTL_SPLLIMULT_M & 0x0F));

    // Enable the peripherals used by this example.
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);

    // Enable clock supply for GPIOC // Add by me
        SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    // Set up the Pin for LED3 // Add by me for blinky
        GPIOPinTypeGPIOOutput(GPIO_PORTC_BASE, GPIO_PIN_7);
        GPIOPinWrite(GPIO_PORTC_BASE, GPIO_PIN_7, ~0);

    // Give C28 control of Port C pin 6 // blinking in 28
        GPIOPinConfigureCoreSelect(GPIO_PORTC_BASE, 0x40, GPIO_PIN_C_CORE_SELECT);

    // Disable clock supply for the watchdog modules  // Add by me
        SysCtlPeripheralDisable(SYSCTL_PERIPH_WDOG1);
        SysCtlPeripheralDisable(SYSCTL_PERIPH_WDOG0);

    // Enable processor interrupts.
    IntMasterEnable();

    //FIXME initial Flash for test

    memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize);

    //  Call Flash Initialization to setup flash waitstates
    //  This function must reside in RAM
    FlashInit();
    // initial flash end

    // Send boot command to allow the C28 application to begin execution
    IPCMtoCBootControlSystem(CBROM_MTOC_BOOTMODE_BOOT_FROM_FLASH);


    // Set GPIO E4 and E5 as UART pins.
    GPIOPinTypeUART(GPIO_PORTE_BASE, GPIO_PIN_4 | GPIO_PIN_5);
    GPIOPinConfigure(GPIO_PE4_U0RX);
    GPIOPinConfigure(GPIO_PE5_U0TX);

    // Configure the UART for 115,200, 8-N-1 operation.
    UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(SYSTEM_CLOCK_SPEED), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));

    // Enable the UART interrupt.
    IntRegister(INT_UART0, UARTIntHandler);
    IntEnable(INT_UART0);
    UARTIntEnable(UART0_BASE, UART_INT_RX | UART_INT_RT);

    // Loop forever echoing data through the UART.
    while(1)
    {
    	if( (byteData != 0x55) && (UARTCharsAvail(UART0_BASE)== 1) )
    	{

    		 byteData = UARTCharGetNonBlocking(UART0_BASE);

    	}

    	if( (byteData == 0x55) && (UARTCharsAvail(UART0_BASE)== 1) )
    	{
    	     wordData =  UARTCharGetNonBlocking(UART0_BASE);
    	     byteData = 0;
    	}

    	if (wordData == 0xAA)
    	{
    		wordData = 0;
    		(*Update)();
    	}
    	//*****************************************************************************
    	// This is add by me for blinky the LED3 ::delay so much, so if data transmit
    	// from PC so fast, UART will not receive in time.
    	//*****************************************************************************
    	        // Turn on the LED.
    	        //
    	       GPIOPinWrite(GPIO_PORTC_BASE, GPIO_PIN_7, 0);
    	        //
    	        // Delay for a bit.
    	        //
    	        for(ulLoop = 0; ulLoop < 800000; ulLoop++)
    	        {
    	        }
    	        //
    	        // Turn off the LED.
    	        //
    	        GPIOPinWrite(GPIO_PORTC_BASE, GPIO_PIN_7, ~0);
    	        //
    	        // Delay for a bit.
    	        //
    	        for(ulLoop = 0; ulLoop < 800000; ulLoop++){}

       //*****************************************************************************

    }
}



