/*--------------------------------------------------------*/
/* Fapi_UserDefinedFunctions.c                            */
/*                                                        */
/* Copyright (c) 2009-2011 Texas Instruments Incorporated */
/*                                                        */
/* F021 Flash API v1.00                                   */
/*--------------------------------------------------------*/

/*!
    \file Fapi_UserDefinedFunctions.c
    \brief Contains all user defined functions that the Fapi functions use.
*/

/* Uncomment the appropriate include file for your device */



/*#include "F021_FMC_BE.h"*/
/*#include "F021_FMC_LE.h"*/
/*#include "F021_Concerto_C28x.h"*/
#include "F021_Concerto_Cortex.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/uart.h"

Fapi_StatusType Fapi_serviceWatchdogTimer(void)
{
   /* User to add their own watchdog servicing code here */

   return(Fapi_Status_Success);
}

Fapi_StatusType Fapi_setupEepromSectorEnable(void)
{
   Fapi_GlobalInit.m_poFlashControlRegisters->Fbse.u32Register = 0xFFFF;   /* Value must be 0xFFFF to enable erase and programming of the EEPROM bank, 0 to disable */
   Fapi_GlobalInit.m_poFlashControlRegisters->FsmSector1.u32Register = 0;  /* Enables/disables sectors  0-31 for bank and sector erase */
   Fapi_GlobalInit.m_poFlashControlRegisters->FsmSector2.u32Register = 0;  /* Enables/disables sectors 32-63 for bank and sector erase */

   return(Fapi_Status_Success);
}

Fapi_StatusType Fapi_setupBankSectorEnable(void)
{
   Fapi_GlobalInit.m_poFlashControlRegisters->FsmSector.FSM_SECTOR_BITS.SECT_ERASED = 0;  /* Enables/disables sectors 0-15 for bank and sector erase */
   Fapi_GlobalInit.m_poFlashControlRegisters->Fbse.u32Register = 0xFFFF;                  /* Enable sectors 0-15 for erase and programming */

   return(Fapi_Status_Success);
}

//******************************************************************************
// user define function :: UART receive or transmit data
//******************************************************************************
uint16 UART_GetWordData()
{
   uint16 wordData;
   uint16 byteData;

   wordData = 0x0000;
   byteData = 0x0000;

   // Fetch the MSB from the host

   while(UARTCharsAvail(UART0_BASE)!= 1) { }
   byteData =  UARTCharGetNonBlocking(UART0_BASE);


   // Fetch the LSB from the host
   while(UARTCharsAvail(UART0_BASE)!= 1) { }
   wordData =  UARTCharGetNonBlocking(UART0_BASE);


   // form the wordData from the MSB:LSB
   wordData |= (byteData << 8);

   return wordData;
}

//*****************************************************************************
// Send a string to the UART.
//*****************************************************************************
void
UARTSend(const unsigned char *pucBuffer, unsigned long ulCount)
{
    // Loop while there are more characters to send.
    while(ulCount--)
    {
        // Write the next character to the UART.
        UARTCharPutNonBlocking(UART0_BASE, *pucBuffer++);
    }
}

//*****************************************************************************
// Get the entry address
//*****************************************************************************
uint32 GetLongData()
{
    uint32 longData;

    // Fetch the upper 1/2 of the 32-bit value
    longData = ( (uint32)(UART_GetWordData)() << 16);

    // Fetch the lower 1/2 of the 32-bit value
    longData |= (uint32)(UART_GetWordData)();

    return longData;
}


uint16 GetData(uint8 *BlockBuf,uint16 Length)
{
	int i;
	uint16 checksum=0;
	for(i=0;i<Length;i++)
	{
	  while(UARTCharsAvail(UART0_BASE)!= 1) { }
	  BlockBuf[i]=UARTCharGetNonBlocking(UART0_BASE);
	  checksum += BlockBuf[i];
	}
  // return STATUS_SUCCESS;
	return checksum;
}

uint16 GetWData(uint16 *BlockBuf,uint16 Length)
{
	int i;
	uint16 temp=0;
	uint16 checksum=0;
    for(i=0;i<Length/2;i++)
    {
    	BlockBuf[i]=UART_GetWordData();
        // calculate checksum
    	temp = ((BlockBuf[i]&0xFF00)>>8);
    	checksum+=temp;
    	temp = (BlockBuf[i]&0xFF);
        checksum+=temp;
    }
    return checksum;
}

/* End of File */

