//###########################################################################
// FILE:   flash_m3.c
// TITLE:  flash API example.
//###########################################################################

#include <string.h>

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_ipc.h"
#include "inc/hw_flash.h"
//#include "inc/hw_memmap.h"
#include "driverlib/debug.h"
#include "driverlib/ipc.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/flash.h"
#include "driverlib/device.h"
#include "driverlib/dcsm.h"
#include "F28M35x_FlashAPIexample_Dual_m3.h"
#include "driverlib/gpio.h"
#include "driverlib/uart.h"
// add by ipc communication
#include "inc/hw_ram.h"
#include "inc/hw_ipc.h"
#include "driverlib/ipc.h"
#include "driverlib/ram.h"
#include "utils/ustdlib.h"

//*****************************************************************************
// FILE Flash API include file
//*****************************************************************************
#include "F021_CONCERTO_Cortex.h"
#include "UserDefinedFunctions.h" // this header should be include under "F021...h"
                                  // or it will cause run fail.
//*****************************************************************************
//! \addtogroup dual_example_list
//! <h1>Flash (flash)</h1>
//!
//! This example illustrates the usage of Flash API to erase and program the
//! flash.
//! Flash API functions are executed from RAM.  See linker command file for
//! details on loading the Flash API in Flash and executing it from RAM.
//!
//! To use the Flash API
//! 1.  If passwords are programmed in Flash, users are supposed to unlock the
//!     flash using the DCSMUnlockZone1CSM and DCSMUnlockZone2CSM functions with
//!     appropriate key if executing Flash API from unsecured memory.  See
//!     security chapter in TRM for details
//! 2.  Users should configure the desired system frequency
//! 3.  Users should configure waitstates for FSM operations based on desired
//!     system frequency using the formula: RWAIT = (SYSCLK/(2*24MHz))-1
//! 4.  Users should include F021_CONCERTO_Cortex.h in the application
//! 5.  Users should include Flash API library to the project
//! 6.  Watchdog is disabled in this project.  If users want to enable and
//!     service the watchdog, users have to include the watchdog service code
//!     in Fapi_UserDefinedFunctions.c file
//! 7.  This example uses F28M35x_FlashAPIexample_Dual_m3.h file
//!
//! Steps to convert code to run out of Flash instead of RAM are as follows: \n
//! 1. Change the command linker file to Flash linker file (i.e. F28M35x.cmd) \n
//! 2. Make sure initialized sections are mapped to Flash. \n
//! 3. Make sure ResetISR() or branch to starting point of your application is
//! mapped to "BOOT_TO_FLASH" address of 0x00200030. (0x13FFF0 on C28) \n
//! 4. Add security files /driverlib/dcsm_z1_secvalues.asm and/or
//! driverlib/dcsm_z2_secvalues.asm to program Z1 and Z2 security values. \n
//! 5. For best performance from the flash, modify the waitstates
//! and enable the flash pipeline as shown in this example. \n
//! Note: any code that manipulates the flash waitstate and pipeline
//! control must be run from RAM. Thus these functions are located
//! in their own memory section called ramfuncs. \n
//! 6. Code that must execute faster than Flash allows can also be
//! run from RAM (i.e. interrupt service routines). \n
//! 7. To copy code from Flash to RAM during runtime, set up .cmd file
//! appropriately and call MemCopy() function (in memcopy.c in utils) \n
//! 8. If using the IntRegister()/IntUnRegister() driverlib functions, replace
//! with IntRAMRegister()/IntRAMUnregister() functions which copy the Flash
//! interrupt vector table into RAM .vtable location. \n
//
//*****************************************************************************

//Lengths of sectors
#define Bzero_16KSector_u32length   0x1000
#define Bzero_64KSector_u32length   0x4000

//Data/Program Buffer used for testing the flash API functions
#define  BYTES_IN_FLASH_BUFFER    0xFF   // Programming data buffer, bytes
uint8    Buffer[BYTES_IN_FLASH_BUFFER];

// define for update APP
#define MaxSize  2048
uint8 BlockBuf[MaxSize];
uint8 BlockBuf1[24];


#pragma DATA_SECTION(MTOCBlockBuf,"SHARERAMS0");
uint16 MTOCBlockBuf[MaxSize/2];


//#define M3App (void   (*)(void))0x00200031
unsigned char  *MarkPtr   =   (unsigned  char *)0x00203900;  //(unsigned  char *)0x00203800;
//#define APP (void   (*)(void))0x00204001

// add by ipc
#define M3_CTOM_PASSMSG   0x2007F7E8            // CTOM MSG RAM offsets for
                                                // passing addresses
#define IPC_Update_COM   0x00003238 // Used for update command to c28
// Global variable used in this example to track errors
unsigned short ErrorFlag;

//*****************************************************************************
// Flags that contain the current value of the interrupt indicator as displayed
// on the CSTN display.
//*****************************************************************************
unsigned long g_ulFlags;

// Structure CSMPSWKEY is defined in dcsm.h
//Users should modify this Zone1 key if the password locations in Flash are
// not all 0xFFFFFFFFs
CSMPSWDKEY Zone1KEY = {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF};
//Users should modify this Zone2 key if the password locations in Flash are
// not all 0xFFFFFFFFs
CSMPSWDKEY Zone2KEY = {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF};

// These are defined by the linker (see device linker command file)
extern unsigned long RamfuncsLoadStart;
extern unsigned long RamfuncsRunStart;
extern unsigned long RamfuncsLoadSize;

//*****************************************************************************
// The error routine that is called if the driver library encounters an error.
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
}

#endif

//******************************************************************************
// For this example, if an error is found just stop here
//******************************************************************************
#pragma CODE_SECTION(Example_Error,"ramfuncs");
void Example_Error(Fapi_StatusType status)
{
    //  Error code will be in the status parameter
//	uint16 i=0;
//	uint16 j=0;
//	j = Fapi_Error_AsyncIncorrectDataBufferLength;
//	i =  status;
        for(;;);
}


//******************************************************************************
//  For this example, once we are done just stop here
//******************************************************************************
#pragma CODE_SECTION(Example_Done,"ramfuncs");
void Example_Done(void)
{
    for(;;);
}

//******************************************************************************
//  M3_CallFlashAPI
//
//  This function will interface to the flash API.
//  Flash API functions used in this function are executed from RAM
//******************************************************************************

void M3_CallFlashAPI(void)
{
    uint32 u32Index = 0;
    uint16 i = 0;
    uint16 bank_width = 0;

    Fapi_StatusType            oReturnCheck;
    Fapi_LibraryInfoType       oLibInfo;
    Fapi_FlashStatusType       oFlashStatus;
    Fapi_FlashBankSectorsType  oFlashBankSectors;
    Fapi_FlashStatusWordType   oFlashStatusWord;

    uint32 BlockEntryPoint;
    uint16  BlockLength,BlockLength1;
    uint16  PCchecksum, M3checksum;
    uint8 Mark=0xF1;// F1 fail; F0 success
    uint8 Mark1=0xF0;
    // for address align
    uint32 Address;
    uint16 BlockLength2;
    uint16 j=0;
    uint16 flag = 0;
    uint16 k=0;
    // This function is required to initialize the Flash API based on System
    // frequency before any other Flash API operation can be performed
    #if CPU_FRQ_100MHZ
    oReturnCheck = Fapi_initializeAPI(F021_CPU0_BASE_ADDRESS, 100);
    #endif

    #if CPU_FRQ_75MHZ
    oReturnCheck = Fapi_initializeAPI(F021_CPU0_BASE_ADDRESS, 75);
    #endif

    #if CPU_FRQ_60MHZ
    oReturnCheck = Fapi_initializeAPI(F021_CPU0_BASE_ADDRESS, 60);
    #endif

    if(oReturnCheck |= Fapi_Status_Success)
    {
        //Check Flash API documentation for possible errors
        Example_Error(oReturnCheck);
    }

    // Fapi_getLibraryInfo function can be used to get the information
    // specific to the compiled version of the API library
    oLibInfo = Fapi_getLibraryInfo();

    // Before performing FSM operations, set the waitstates for FSM
    // operations calculated using RWAIT = (SYSCLK/(2*24MHz))-1
    // If RWAIT results in a fractional value, round it up to the nearest
    // integer.
    // Please note that RWAIT for read operation should be calculated
    // differently.  See Internal memory guide section in TRM
    #if CPU_FRQ_100MHZ
    HWREG(FLASH_CONFIG_BASE + FLASHCTRL_O_FRDCNTL) = (2 << FLASHCTRL_FRDCNTL_RWAIT_S);
    #endif

    #if CPU_FRQ_75MHZ
    HWREG(FLASH_CONFIG_BASE + FLASHCTRL_O_FRDCNTL) = (1 << FLASHCTRL_FRDCNTL_RWAIT_S);
    #endif

    #if CPU_FRQ_60MHZ
    HWREG(FLASH_CONFIG_BASE + FLASHCTRL_O_FRDCNTL) = (1 << FLASHCTRL_FRDCNTL_RWAIT_S);
    #endif

    // Fapi_setActiveFlashBank function sets the Flash bank and FMC
    // for further Flash operations to be performed on the bank
    oReturnCheck = Fapi_setActiveFlashBank(Fapi_FlashBank0);
    if(oReturnCheck |= Fapi_Status_Success)
    {
        // Check Flash API documentation for possible errors
        Example_Error(oReturnCheck);
    }

    // Fapi_getBankSectors function returns the bank starting address,
    // number of sectors, sector sizes, and bank technology type
    // Above information is returned in a structure oFlashBankSectors of type
    // Fapi_FlashBankSectorsType
    oReturnCheck = Fapi_getBankSectors(Fapi_FlashBank0,&oFlashBankSectors);
    if(oReturnCheck |= Fapi_Status_Success)
    {
        // Check Flash API documentation for possible errors
        Example_Error(oReturnCheck);
    }
/*for test
    if (*(MarkPtr-16) == 0xF1)
        {
        	u32Index = (uint32)(MarkPtr-15);
        	oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,&Mark1,
        	    					        1,
        	    			    		    0,
        	    			    		    0,
        	    			    		    Fapi_AutoEccGeneration); //Fapi_AutoEccGeneration //Fapi_DataOnly
        }*/
//*****************************************************************************************
    // before erase app flash, program a mark in CKFA sector
    if (*MarkPtr != 0xF1)
    {
    	u32Index = (uint32)MarkPtr;
    	oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,&Mark,
    	    					        1,
    	    			    		    0,
    	    			    		    0,
    	    			    		    Fapi_AutoEccGeneration); //Fapi_AutoEccGeneration //Fapi_DataOnly
    }


//*****************************************************************************************
// Erase FLASH sector M L K, that is APP should be in this sectors
//*****************************************************************************************
    // Erase Sector L
    // Sector N has the CKFA code so leave it programmed
    oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                   (uint32 *)Bzero_SectorL_start);

    // Wait until FSM is done with erase sector operation
    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}

    // Verify that Sector L is erased.  The Erase step itself does a verify
    // as it goes.  This verify is a 2nd verification that can be done.
    oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorL_start,
                   Bzero_16KSector_u32length,
                   &oFlashStatusWord);
    if(oReturnCheck |= Fapi_Status_Success)
    {
        //Check Flash API documentation for possible errors
        //If Erase command fails, use Fapi_getFsmStatus() function to get
        // the FMSTAT register contents
        //to see if any of the EV bit, ESUSP bit, CSTAT bit or VOLTSTAT
        //bit is set (Refer to API documentation for more details)
        Example_Error(oReturnCheck);
    }

    // Erase Sector K

    oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                   (uint32 *)Bzero_SectorK_start);

    // Wait until FSM is done with erase sector operation
    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}

    // Verify that SectorK is erased.  The Erase step itself does a verify as
    // it goes.  This verify is a 2nd verification that can be done.
    oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorK_start,
                   Bzero_16KSector_u32length,
                   &oFlashStatusWord);

    if(oReturnCheck |= Fapi_Status_Success)
    {
        // Check Flash API documentation for possible errors
        // If Erase command fails, use Fapi_getFsmStatus() function to get
        // the FMSTAT register contents
        // to see if any of the EV bit, ESUSP bit, CSTAT bit or VOLTSTAT bit is
        // set (Refer to API documentation for more details)
        Example_Error(oReturnCheck);
    }

    // Erase Sector M
       oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                      (uint32 *)Bzero_SectorM_start);

       // Wait until FSM is done with erase sector operation
       while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}

       // Verify that SectorK is erased.  The Erase step itself does a verify as
       // it goes.  This verify is a 2nd verification that can be done.
       oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorM_start,
                      Bzero_16KSector_u32length,
                      &oFlashStatusWord);

       if(oReturnCheck |= Fapi_Status_Success)
       {
           // Check Flash API documentation for possible errors
           // If Erase command fails, use Fapi_getFsmStatus() function to get
           // the FMSTAT register contents
           // to see if any of the EV bit, ESUSP bit, CSTAT bit or VOLTSTAT bit is
           // set (Refer to API documentation for more details)
           Example_Error(oReturnCheck);
       }
// End erase
//*****************************************************************************************

    // Send "M3" to PC indicate that M3 is ready for receive update APP date
    while(UARTBusy(UART0_BASE)){}
    UARTSend((unsigned char *)"M3", 2);
    // Receive block data and program to flash
    while(1)
    {
    	 // Get the 4 byte 32 bits block entry address
    	 BlockEntryPoint = GetLongData();
    	 // if entry address is as follow, then update is finish
    	 if(BlockEntryPoint==0x55555555)
    		{
    		 if(flag ==1)
    		 {
    			 oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)Address,BlockBuf1,
    			     					     					        BlockLength2,
    			     					     			    		    0,
    			     					     			    		    0,
    			     					     			    		    Fapi_AutoEccGeneration);
    		 }
    		 if (*MarkPtr == 0xF1)
    		     {
    		     	u32Index = (uint32)MarkPtr+8;
    		     	oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,&Mark1,
    		     	    					        1,
    		     	    			    		    0,
    		     	    			    		    0,
    		     	    			    		    Fapi_AutoEccGeneration); //Fapi_AutoEccGeneration //Fapi_DataOnly
    		     }
    		 //TODO clear updata mark save in address 0x3E8000 ::clear the mark
    		 // Status = Flash_Erase(SECTORD,&FlashStatus);
    		 // if(Status != STATUS_SUCCESS)
    		 //    {
    		 //      Example_Error(Status);
    		 //    }
    	    break;
    		}
    	 // Get the 2 byte length
    	 BlockLength = UART_GetWordData();
    	 BlockLength1= BlockLength;
    	 // copy data from PC to RAM through UART,and return checksum calculate by M3
    	 M3checksum = GetData(BlockBuf,BlockLength);
    	 // get the check sum from pc calculation
    	 PCchecksum = UART_GetWordData();
    	 //
         if(M3checksum == PCchecksum)   // if M3checksum is equal to PCchecksum, then program
         {
    	 // Disable ECC so that error is not generated when reading Flash
    	     // contents without ECC
    	 HWREG(FLASH_ERROR_BASE + FLASHERR_O_ECC_ENABLE) = 0x0;

    	 // program the update block data to FLASH
    	 for(i=0, u32Index = BlockEntryPoint;
    	       (u32Index < (BlockEntryPoint + BlockLength))
    	       && (oReturnCheck == Fapi_Status_Success); /*i+= 16, u32Index+= 16*/)
    	 {
    		 if(BlockLength1 < 16)
    		 {
    			 flag = 1;
    			 Address = u32Index;
    			 BlockLength2 = BlockLength1;
    			 for(j=0;j<BlockLength1;j++)
    			 {
    				 BlockBuf1[j]=BlockBuf[i];
    				 i++;
    			 }
    			 u32Index+=BlockLength1;
    			 /*
    			 oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,BlockBuf+i,
    					        BlockLength1,
    			    		    0,
    			    		    0,
    			    		    Fapi_AutoEccGeneration); //Fapi_AutoEccGeneration //Fapi_DataOnly
    			    		    */
    		 }
    		 else
    		 {
    			 bank_width = u32Index & 0x0000000F;
    			// if(bank_width > 0)
    			// {
    				 if(flag ==1 && ((Address & 0xFFFFFFF0) == (u32Index & 0xFFFFFFF0)) )
    				 {
    						for(k=j;k<(j+(u32Index-Address-BlockLength2));k++)
    						{
    							BlockBuf1[k]=0xFF;
    						}
    						for(;k<16;k++)
    						{
    							BlockBuf1[k]=BlockBuf[i];
    							i++;
    						}
    						oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)Address,BlockBuf1,
    						    			     				 16,  //FIXME
    						    			     				 0,
    						    			     				 0,
    						    			     				 Fapi_AutoEccGeneration);//Fapi_AutoEccGeneration
    						 u32Index += (16-bank_width);
    						 while(Fapi_checkFsmForReady() == Fapi_Status_FsmBusy);
    						 BlockLength1 -= (16-bank_width);
    						// i += (16-bank_width);
    						 flag = 0;
    				 }
    				 else
    				 {
    					 if(flag ==1)
    					 { oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)Address,BlockBuf1,
    					     					        BlockLength2,
    					     			    		    0,
    					     			    		    0,
    					     			    		    Fapi_AutoEccGeneration);
    					   flag = 0;
    					 }

    			     oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,BlockBuf+i,
    			     				 16-bank_width,
    			     				 0,
    			     				 0,
    			     				 Fapi_AutoEccGeneration);//Fapi_AutoEccGeneration
    			     u32Index += (16-bank_width);
    			     while(Fapi_checkFsmForReady() == Fapi_Status_FsmBusy);
    			     BlockLength1 -= (16-bank_width);
    			     i += (16-bank_width);
    				 }

    			// }
    		   /*  oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,BlockBuf+i,
    		                    16,
    		                    0,
    		                    0,
    		                    Fapi_AutoEccGeneration);//Fapi_AutoEccGeneration*/
    		 }

    		 while(Fapi_checkFsmForReady() == Fapi_Status_FsmBusy);

    		 if(oReturnCheck |= Fapi_Status_Success)
    		 {
    		    // Check Flash API documentation for possible errors
    		    Example_Error(oReturnCheck);
    		 }

    		 // Read FMSTAT register contents to know the status of FSM after
    		 // program command for any debug
    		 oFlashStatus = Fapi_getFsmStatus();
    		// if(BlockLength1 > 8)
    	//	 BlockLength1 -= 16;

    	 }

        // Send "ok" to PC indicate that M3 is ready for receive another update APP block
         while(UARTBusy(UART0_BASE)){}
         UARTSend((unsigned char *)"ok", 2);
       }// if checksum equal end
         else// else if checksum is not equal,then send "no" to PC, tell PC to send data block again
         {
           while(UARTBusy(UART0_BASE)){}
           UARTSend((unsigned char *)"no", 2);
         } // checksum detection end
    } //end while, that is to say, update is finish

 /*
    if(oReturnCheck |= Fapi_Status_Success)
        		 {
        		    // Check Flash API documentation for possible errors
        		    Example_Error(oReturnCheck);
        		 }
*/
    // Enable ECC
        HWREG(FLASH_ERROR_BASE + FLASHERR_O_ECC_ENABLE) = 0xA;

        // Leave control over pump
        FlashLeavePump();



   // jump to APP Flash entry point
   // (*M3App)();
}


void M3_CallC28FlashAPI(void)
{
	unsigned short usWWord16;
    uint32 BlockEntryPoint;
    uint16  BlockLength;
    uint16 index;
    uint16  PCchecksum, C28checksum;

    // initial MTOCBlockBuf
    for(index = 0; index < MaxSize; index++)
    {
      MTOCBlockBuf[index]=0;
    }




//*****************************************************************************
    // wait until IPC M to C request or status flags are available.
    while (HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCFLG) & (IPC_FLAG1 | IPC_FLAG32)){}

    // write update command for 28 to read and update
    HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCCOM) = IPC_Update_COM;
    // Force IPC event on selected request task and enable status-checking
    HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCSET) |= (IPC_FLAG1 | IPC_FLAG32);

    // wait until 28 use IPC flag17 to tell m3 that it's ready for update
  //  while (!IPCCtoMFlagBusy(IPC_FLAG17));
  //  IPCCtoMFlagAcknowledge(IPC_FLAG17);
    // wait until C28 acknowledge flag1 and flag32
    while (HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCFLG) & (IPC_FLAG1 | IPC_FLAG32)){}

    // Send "M3" to PC indicate that 28 is ready for receive update APP date
    while(UARTBusy(UART0_BASE)){}
    UARTSend((unsigned char *)"28", 2);

    while(1)
    {
   	 // Get the 4 byte 32 bits block entry address
   	 BlockEntryPoint = GetLongData();
   	 // if entry address is as follow, then update is finish
   	 if(BlockEntryPoint==0x55555555)
   		{
   		 //TODO clear update mark save in address 0x3E8000 ::clear the mark
   		 // Status = Flash_Erase(SECTORD,&FlashStatus);
   		 // if(Status != STATUS_SUCCESS)
   		 //    {
   		 //      Example_Error(Status);
   		 //    }
   		HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCCOM) = IPC_DATA_WRITE_16;
   		HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCADDR) = 0x55555555;
   		HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCSET) |= (IPC_FLAG1 | IPC_FLAG32);
   	    break;
   		}
   	 // Get the 2 byte length
   	 BlockLength = UART_GetWordData();
   	 // copy 16-bit data from PC to RAM through UART,and calculate checksum of receive data
   	 C28checksum = GetWData(MTOCBlockBuf,BlockLength);
   	// get the check sum from pc calculation
   	 PCchecksum = UART_GetWordData();
   	//
   	  if(C28checksum == PCchecksum) // if C28checksum is equal to PCchecksum, then send IPC to tell 28 for update
   	  {

   	 // Transmit block entry point and length to 28, then 28 can read buffer and call API
   	 // FLAG1 and FLAG32 are used in this IPC
   	 usWWord16 = BlockLength/2;  // as 28buffer is 16-bit
    // pulEntryPoint = (void *)BlockEntryPoint;

     IPCLiteMtoCDataWrite(IPC_FLAG1, BlockEntryPoint,(unsigned long)usWWord16,
                          IPC_LENGTH_16_BITS,
                          IPC_FLAG32);
/*
     HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCCOM) = IPC_DATA_WRITE_16;
     HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCADDR) = BlockEntryPoint;
     HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCDATAW) = usWWord16;
     HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCSET) |= (IPC_FLAG1 | IPC_FLAG32);
*/
     // wait until 28 finish program and clear IPC flag
     while (HWREG(MTOCIPC_BASE + IPC_O_MTOCIPCFLG) & (IPC_FLAG1 | IPC_FLAG32)){}

     // Send "ok" to PC indicate that M3 is ready for receive another update APP block
     while(UARTBusy(UART0_BASE)){}
     UARTSend((unsigned char *)"ok", 2);
   	  }
   	  else // if checksum is wrong, then send "no" to PC, tell PC to send data block again
   	  {
   		while(UARTBusy(UART0_BASE)){}
   	    UARTSend((unsigned char *)"no", 2);
   	  }
    }//while end

}



//*****************************************************************************************
// This is an example code demonstrating F021 Flash API usage.  This code is all in Flash.
//*****************************************************************************************
int
main(void)
{
	//  MTOCBlockBuf[0]=0;
	uint16 UpdateCore;
	uint16 wordData1 = 0x0000;
	uint16 byteData1 = 0x0000;
    // Disable Protection
    HWREG(SYSCTL_MWRALLOW) =  0xA5A5A5A5;

    // Setup main clock tree for 75MHz - M3 and 150MHz - C28x
    SysCtlClockConfigSet(SYSCTL_SYSDIV_1 | SYSCTL_M3SSDIV_2 | SYSCTL_USE_PLL |
                             (SYSCTL_SPLLIMULT_M & 0x0F));

    // Initialize M3toC28 message RAM and Sx SARAM and wait until initialized
    //
    RAMMReqSharedMemAccess (S0_ACCESS, SX_M3MASTER);

    //  Unlock Zone1 and Zone2 CSM.
    //  If the API functions are going to run in unsecured RAM
    //  then the CSM must be unlocked in order for the flash
    //  API functions to access the flash.
    //  If the flash API functions are executed from secure memory
    //  then this step is not required.

    DCSMUnlockZone1CSM (&Zone1KEY);
    DCSMUnlockZone2CSM (&Zone2KEY);

    //  Copy API Functions and required user interface functions to RAM.
    //
    //  The flash API functions MUST be run out of RAM memory.
    //  If the Flash API is already in RAM then this step
    //  can be skipped.
    //  DO NOT run the Flash API from Flash

    memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize);

    //  Call Flash Initialization to setup flash waitstates
    //  This function must reside in RAM
    FlashInit();

    // Disable clock supply for the watchdog modules
    SysCtlPeripheralDisable(SYSCTL_PERIPH_WDOG1);
    SysCtlPeripheralDisable(SYSCTL_PERIPH_WDOG0);

    // Gain flash pump semaphore
    // This function must reside in RAM
    FlashGainPump();

    //*****************************************************************************
    // FIXME initial SCI
    //*****************************************************************************
    // Enable the peripherals used by this example.
     SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
     SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
     // Set GPIO E4 and E5 as UART pins.
     GPIOPinTypeUART(GPIO_PORTE_BASE, GPIO_PIN_4 | GPIO_PIN_5);
     GPIOPinConfigure(GPIO_PE4_U0RX);
     GPIOPinConfigure(GPIO_PE5_U0TX);

     // Configure the UART for 115,200, 8-N-1 operation.
     UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(SYSTEM_CLOCK_SPEED), 115200,
                         (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                          UART_CONFIG_PAR_NONE));

     //
     while(*MarkPtr != 0xFF) // update success last time, fine a unit to be FF or F1
    		{
     			MarkPtr +=8;
     		}
     if (*(MarkPtr-8) == 0xF1) // program fail, re-update
     {
    	 while(1) // if re-update, wait for correct update command
    	    {
    	    	if( (byteData1 != 0x55) && (UARTCharsAvail(UART0_BASE)== 1) )
    	    	{

    	    		 byteData1 = UARTCharGetNonBlocking(UART0_BASE);

    	    	}

    	    	if( (byteData1 == 0x55) && (UARTCharsAvail(UART0_BASE)== 1) )
    	    	{
    	    	     wordData1 =  UARTCharGetNonBlocking(UART0_BASE);
    	    	     byteData1 = 0;
    	    	}

    	    	if (wordData1 == 0xAA)
    	    	{
    	    		wordData1 = 0;
    	    		break; // if update command is 0x55AA, then break this while
    	    	}
    	     }
     }
     // debug: if(UART_GetWordData() == 0x55AA)
     // Send ok to PC indicate that PC connect target
     UARTSend((unsigned char *)"ok", 2);

     // ACK to PC core selection, if PC send M3, when finish erase FLASH, return M3, vice versa 28.
     UpdateCore = UART_GetWordData();
     if(UpdateCore == 0x4d33)   // "M3"=0x4d33
     {
      //  Jump to RAM and call the Flash API functions
      //  communicate PC through UART to receive date for update, call flash API to program
           M3_CallFlashAPI();
      // after M3 finish update, call software reset
           SysCtlReset();
     }

     else if(UpdateCore == 0x3238) //"28"=0x3238
     {
    	//jump to 28 Flash update process
    	 FlashLeavePump();
    	 M3_CallC28FlashAPI();
    	// after 28 finish update, call M3 update if needed
    	 if(UART_GetWordData()== 0x4d33) // M3 will wait here to receive data from PC
    	 {		                         // so if want device to run, there should be reboot.
    		 FlashGainPump();
    		 M3_CallFlashAPI();
    	 }
    	 SysCtlReset();    // software reset the whole device here
     }

     // The M3 or 28 FLASH update code should take control, so this should never be reached.
     // Just in case, loop forever.
     else{}
     while(1){}
}







